@@ -5,7 +5,7 @@
 #include "newt.h"
 #include "newt_pr.h"
 
-enum type { CHECK, RADIO, LISTITEM };
+enum type { CHECK, RADIO };
 
 struct checkbox {
     char * text;
@@ -17,9 +17,9 @@ struct checkbox {
     int active, inactive;
     const void * data;
     int flags;
+    int hasFocus;
 };
 
-static void cbDrawIt(newtComponent c, int active);
 static void makeActive(newtComponent co);
 
 static void cbDraw(newtComponent c);
@@ -34,38 +34,6 @@ static struct componentOps cbOps = {
     newtDefaultMappedHandler,
 } ;
 
-newtComponent newtListitem(int left, int top, const char * text, int isDefault,
-			      newtComponent prevItem, const void * data, int flags) {
-    newtComponent co;
-    struct checkbox * li;
-
-    co = newtRadiobutton(left, top, text, isDefault, prevItem);
-    li = co->data;
-    li->type = LISTITEM;
-    li->flags = flags & (NEWT_FLAG_RETURNEXIT);
-    li->inactive = COLORSET_LISTBOX;
-    li->active = COLORSET_ACTLISTBOX;
-    li->data = data;
-
-    return co;
-}
-
-void * newtListitemGetData(newtComponent co) {
-    struct checkbox * rb = co->data;
-
-    return (void *)rb->data;
-}
-
-void newtListitemSet(newtComponent co, const char * text) {
-    struct checkbox * li = co->data;
-
-    free(li->text);
-    li->text = strdup(text);
-
-    if (strlen(text) + 4 > (unsigned int)co->width)
-	co->width = strlen(text) + 4;
-}
-
 newtComponent newtRadiobutton(int left, int top, const char * text, int isDefault,
 			      newtComponent prevButton) {
     newtComponent co;
@@ -94,7 +62,7 @@ newtComponent newtRadiobutton(int left, int top, const char * text, int isDefaul
 
 newtComponent newtRadioGetCurrent(newtComponent setMember) {
     struct checkbox * rb = setMember->data;
-    
+
     setMember = rb->lastButton;
     rb = setMember->data;
 
@@ -112,6 +80,13 @@ char newtCheckboxGetValue(newtComponent co) {
     return cb->value;
 }
 
+void newtCheckboxSetValue(newtComponent co, char value) {
+    struct checkbox * cb = co->data;
+
+    *cb->result = value;
+    cbDraw(co);
+}
+
 newtComponent newtCheckbox(int left, int top, const char * text, char defValue,
 			   const char * seq, char * result) {
     newtComponent co;
@@ -131,6 +106,7 @@ newtComponent newtCheckbox(int left, int top, const char * text, char defValue,
     cb->text = strdup(text);
     cb->seq = strdup(seq);
     cb->type = CHECK;
+    cb->hasFocus = 0;
     cb->inactive = COLORSET_CHECKBOX;
     cb->active = COLORSET_ACTCHECKBOX;
     defValue ? (*cb->result = defValue) : (*cb->result = cb->seq[0]);
@@ -147,19 +123,36 @@ newtComponent newtCheckbox(int left, int top, const char * text, char defValue,
     return co;
 }
 
-static void cbDraw(newtComponent c) {
-    cbDrawIt(c, 0);
+void newtCheckboxSetFlags(newtComponent co, int flags, enum newtFlagsSense sense) {
+    struct checkbox * cb = co->data;
+    int row, col;
+
+    cb->flags = newtSetFlags(cb->flags, flags, sense);
+
+    if (!(cb->flags & NEWT_FLAG_DISABLED))
+	co->takesFocus = 1;
+    else
+	co->takesFocus = 0;
+
+    newtGetrc(&row, &col);
+    cbDraw(co);
+    newtGotorc(row, col);
 }
 
-static void cbDrawIt(newtComponent c, int active) {
+static void cbDraw(newtComponent c) {
     struct checkbox * cb = c->data;
 
-    if (c->top == -1) return;
+    if (c->top == -1 || !c->isMapped) return;
 
-    if (cb->type == LISTITEM && *cb->result != ' ')
-	SLsmg_set_color(cb->active);
-    else
-	SLsmg_set_color(cb->inactive);
+    if (cb->flags & NEWT_FLAG_DISABLED) {
+	cb->inactive = NEWT_COLORSET_DISENTRY;
+	cb->active = NEWT_COLORSET_DISENTRY;
+    } else {
+	cb->inactive = COLORSET_CHECKBOX;
+	cb->active = COLORSET_ACTCHECKBOX;
+    }
+
+    SLsmg_set_color(cb->inactive);
 
     newtGotorc(c->top, c->left);
 
@@ -178,13 +171,11 @@ static void cbDrawIt(newtComponent c, int active) {
 
     SLsmg_write_string(cb->text);
 
-    if (active) 
+    if (cb->hasFocus)
 	SLsmg_set_color(cb->active);
 
-    if (cb->type != LISTITEM) {
-	newtGotorc(c->top, c->left + 1);
-	SLsmg_write_char(*cb->result);
-    }
+    newtGotorc(c->top, c->left + 1);
+    SLsmg_write_char(*cb->result);
 }
 
 static void cbDestroy(newtComponent co) {
@@ -204,16 +195,14 @@ struct eventResult cbEvent(newtComponent co, struct event ev) {
     if (ev.when == EV_NORMAL) {
 	switch (ev.event) {
 	  case EV_FOCUS:
-	    if (cb->type == LISTITEM)
-		makeActive(co);
-	    else 
-		cbDrawIt(co, 1);
-	    
+	    cb->hasFocus = 1;
+	    cbDraw(co);
 	    er.result = ER_SWALLOWED;
 	    break;
 
 	  case EV_UNFOCUS:
-	    cbDrawIt(co, 0);
+	    cb->hasFocus = 0;
+	    cbDraw(co);
 	    er.result = ER_SWALLOWED;
 	    break;
 
@@ -227,33 +216,50 @@ struct eventResult cbEvent(newtComponent co, struct event ev) {
 			*cb->result = *cb->seq;
 		    else {
 			cur++;
-			if (! *cur) 
+			if (! *cur)
 			    *cb->result = *cb->seq;
 			else
 			    *cb->result = *cur;
 		    }
-		    cbDrawIt(co, 1);
+		    cbDraw(co);
 		    er.result = ER_SWALLOWED;
+
+		    if (co->callback)
+			co->callback(co, co->callbackData);
 		} else {
 		    er.result = ER_IGNORED;
 		}
 	    } else if(ev.u.key == NEWT_KEY_ENTER) {
-		if((cb->type == LISTITEM)
-		   && (cb->flags | NEWT_FLAG_RETURNEXIT)) {
-		    er.u.focus = co;
-		    er.result = ER_EXITFORM;
-		} else
-		    er.result = ER_IGNORED;
+		er.result = ER_IGNORED;
 	    } else {
 		er.result = ER_IGNORED;
 	    }
 	    break;
-	}
-    } else 
-	er.result = ER_IGNORED;
+   	  case EV_MOUSE:
+	    if (ev.u.mouse.type == MOUSE_BUTTON_DOWN) {
+		if (cb->type == RADIO) {
+		    makeActive(co);
+		} else if (cb->type == CHECK) {
+		    cur = strchr(cb->seq, *cb->result);
+		    if (!cur)
+			*cb->result = *cb->seq;
+		    else {
+			cur++;
+			if (! *cur)
+			    *cb->result = *cb->seq;
+			else
+			    *cb->result = *cur;
+		    }
+		    cbDraw(co);
+		    er.result = ER_SWALLOWED;
 
-    if (er.result == ER_SWALLOWED && co->callback)
-	co->callback(co, co->callbackData);
+		    if (co->callback)
+			co->callback(co, co->callbackData);
+		}
+	    }
+	}
+    } else
+	er.result = ER_IGNORED;
 
     return er;
 }
@@ -272,8 +278,11 @@ static void makeActive(newtComponent co) {
     }
     if (curr) {
 	rb->value = rb->seq[0];
-	cbDrawIt(curr, 0);
-    } 
+	cbDraw(curr);
+    }
     cb->value = cb->seq[1];
-    cbDrawIt(co, 1);
+    cbDraw(co);
+
+    if (co->callback)
+	co->callback(co, co->callbackData);
 }
