@@ -7,6 +7,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 
 #include "newt.h"
 #include "newt_pr.h"
@@ -27,7 +28,7 @@ struct listbox {
     int curHeight;	/* size of text w/o border */
     int sbAdjust;
     int bdxAdjust, bdyAdjust;
-    int numItems, numSelected; 
+    int numItems, numSelected;
     int userHasSetWidth;
     int currItem, startShowItem; /* startShowItem is the first item displayed
 				   on the screen */
@@ -45,7 +46,7 @@ static void listboxDestroy(newtComponent co);
 static struct eventResult listboxEvent(newtComponent co, struct event ev);
 static void newtListboxRealSetCurrent(newtComponent co);
 static void listboxPlace(newtComponent co, int newLeft, int newTop);
-static inline void updateWidth(newtComponent co, struct listbox * li, 
+static inline void updateWidth(newtComponent co, struct listbox * li,
 				int maxField);
 static void listboxMapped(newtComponent co, int isMapped);
 
@@ -72,7 +73,8 @@ static void listboxPlace(newtComponent co, int newLeft, int newTop) {
     co->left = newLeft;
 
     if (li->sb)
-	li->sb->ops->place(li->sb, co->left + co->width - 1, co->top);
+	li->sb->ops->place(li->sb, co->left + co->width - li->bdxAdjust - 1,
+			   co->top);
 }
 
 newtComponent newtListbox(int left, int top, int height, int flags) {
@@ -97,28 +99,30 @@ newtComponent newtListbox(int left, int top, int height, int flags) {
     li->sbAdjust = 0;
     li->bdxAdjust = 0;
     li->bdyAdjust = 0;
-    li->flags = flags & (NEWT_FLAG_RETURNEXIT | NEWT_FLAG_DOBORDER | 
+    li->flags = flags & (NEWT_FLAG_RETURNEXIT | NEWT_FLAG_BORDER |
 			 NEWT_FLAG_MULTIPLE);
 
-    if (li->flags & NEWT_FLAG_DOBORDER) {
+    if (li->flags & NEWT_FLAG_BORDER) {
 	li->bdxAdjust = 2;
 	li->bdyAdjust = 1;
     }
 
+    co->height = height;
+    li->curHeight = co->height - (2 * li->bdyAdjust);
+
     if (height) {
 	li->grow = 0;
-	if (flags & NEWT_FLAG_NOSCROLL) {
-	    sb = NULL;
-	} else {
-	    sb = newtVerticalScrollbar(left, top, height, COLORSET_LISTBOX,
-				       COLORSET_ACTLISTBOX);
+	if (flags & NEWT_FLAG_SCROLL) {
+	    sb = newtVerticalScrollbar(left, top + li->bdyAdjust,
+					li->curHeight,
+					COLORSET_LISTBOX, COLORSET_ACTLISTBOX);
 	    li->sbAdjust = 3;
+	} else {
+	    sb = NULL;
 	}
-	co->height = height;
     } else {
 	li->grow = 1;
 	sb = NULL;
-	co->height = 2 * li->bdyAdjust;
     }
 
     li->sb = sb;
@@ -131,18 +135,17 @@ newtComponent newtListbox(int left, int top, int height, int flags) {
     co->callback = NULL;
 
     updateWidth(co, li, 5);
-    li->curHeight = co->height - (2 * li->bdyAdjust);
 
     return co;
 }
 
-static inline void updateWidth(newtComponent co, struct listbox * li, 
+static inline void updateWidth(newtComponent co, struct listbox * li,
 				int maxField) {
     li->curWidth = maxField;
     co->width = li->curWidth + li->sbAdjust + 2 * li->bdxAdjust;
 
     if (li->sb)
-	li->sb->left = co->left + co->width - 1;
+	li->sb->left = co->left + co->width - li->bdxAdjust - 1;
 }
 
 void newtListboxSetCurrentByKey(newtComponent co, void * key) {
@@ -193,7 +196,7 @@ static void newtListboxRealSetCurrent(newtComponent co)
 
 void newtListboxSetWidth(newtComponent co, int width) {
     struct listbox * li = co->data;
-    
+
     co->width = width;
     li->curWidth = co->width - li->sbAdjust - 2 * li->bdxAdjust;
     li->userHasSetWidth = 1;
@@ -215,30 +218,34 @@ void * newtListboxGetCurrent(newtComponent co) {
 	return NULL;
 }
 
-void newtListboxSelectItem(newtComponent co, int item,
+void newtListboxSelectItem(newtComponent co, const void * key,
 	enum newtFlagsSense sense)
 {
     struct listbox * li = co->data;
     int i;
-    struct items *iitem;
-    
-    for(i = 0, iitem = li->boxItems; iitem != NULL && i < item;
-	i++, iitem = iitem->next);
+    struct items * item;
 
-    if (iitem) {
-	if(iitem->isSelected && sense != NEWT_FLAGS_SET)
-	    li->numSelected--;
-	else if(!iitem->isSelected && sense != NEWT_FLAGS_RESET)
-	    li->numSelected++;
-	switch(sense) {
-		case NEWT_FLAGS_RESET:
-			iitem->isSelected = 0; break;
-		case NEWT_FLAGS_SET:
-			iitem->isSelected = 1; break;
-		case NEWT_FLAGS_TOGGLE:
-			iitem->isSelected = !iitem->isSelected;
-	}
+    item = li->boxItems, i = 0;
+    while (item && item->data != key)
+	item = item->next, i++;
+
+    if (!item) return;
+
+    if (item->isSelected)
+	li->numSelected--;
+
+    switch(sense) {
+	case NEWT_FLAGS_RESET:
+		item->isSelected = 0; break;
+	case NEWT_FLAGS_SET:
+		item->isSelected = 1; break;
+	case NEWT_FLAGS_TOGGLE:
+		item->isSelected = !item->isSelected;
     }
+
+    if (item->isSelected)
+	li->numSelected++;
+
     listboxDraw(co);
 }
 
@@ -276,7 +283,7 @@ void ** newtListboxGetSelection(newtComponent co, int *numitems)
     return retval;
 }
 
-void newtListboxSetText(newtComponent co, int num, const char * text) {
+void newtListboxSetEntry(newtComponent co, int num, const char * text) {
     struct listbox * li = co->data;
     int i;
     struct items *item;
@@ -298,10 +305,6 @@ void newtListboxSetText(newtComponent co, int num, const char * text) {
 	listboxDraw(co);
 }
 
-void newtListboxSetEntry(newtComponent co, int num, const char * text) {
-    newtListboxSetText(co, num, text);
-}
-
 void newtListboxSetData(newtComponent co, int num, void * data) {
     struct listbox * li = co->data;
     int i;
@@ -313,7 +316,7 @@ void newtListboxSetData(newtComponent co, int num, void * data) {
     item->data = data;
 }
 
-int newtListboxAddEntry(newtComponent co, const char * text,
+int newtListboxAppendEntry(newtComponent co, const char * text,
 	                const void * data) {
     struct listbox * li = co->data;
     struct items *item;
@@ -331,7 +334,7 @@ int newtListboxAddEntry(newtComponent co, const char * text,
 
     item->text = strdup(text); item->data = data; item->next = NULL;
     item->isSelected = 0;
-    
+
     if (li->grow)
 	co->height++, li->curHeight++;
     li->numItems++;
@@ -371,9 +374,9 @@ int newtListboxInsertEntry(newtComponent co, const char * text,
 
     item->text = strdup(text?text:"(null)"); item->data = data;
     item->isSelected = 0;
-    
+
     if (li->sb)
-	li->sb->left = co->left + co->width - 1;
+	li->sb->left = co->left + co->width - li->bdxAdjust - 1;
     li->numItems++;
 
     listboxDraw(co);
@@ -442,7 +445,7 @@ void newtListboxClear(newtComponent co)
     }
     li->numItems = li->numSelected = li->currItem = li->startShowItem = 0;
     li->boxItems = NULL;
-    if (!li->userHasSetWidth) 
+    if (!li->userHasSetWidth)
 	updateWidth(co, li, 5);
 }
 
@@ -462,7 +465,7 @@ void newtListboxGetEntry(newtComponent co, int num, char **text, void **data) {
     }
 
     i = 0;
-    item = li->boxItems; 
+    item = li->boxItems;
     while (item && i < num) {
 	i++, item = item->next;
     }
@@ -471,7 +474,7 @@ void newtListboxGetEntry(newtComponent co, int num, char **text, void **data) {
 	if (text)
 	    *text = item->text;
 	if (data)
-	    *data = (void *)item->data; 
+	    *data = (void *)item->data;
     }
 }
 
@@ -483,10 +486,7 @@ static void listboxDraw(newtComponent co)
 
     if (!co->isMapped) return ;
 
-    if(li->sb)
-	li->sb->ops->draw(li->sb);
-
-    if(li->flags & NEWT_FLAG_DOBORDER) {
+    if(li->flags & NEWT_FLAG_BORDER) {
       if(li->isActive)
 	  SLsmg_set_color(NEWT_COLORSET_ACTLISTBOX);
       else
@@ -495,8 +495,11 @@ static void listboxDraw(newtComponent co)
       newtDrawBox(co->left, co->top, co->width, co->height, 0);
     }
 
+    if(li->sb)
+	li->sb->ops->draw(li->sb);
+
     SLsmg_set_color(NEWT_COLORSET_LISTBOX);
-    
+
     for(i = 0, item = li->boxItems; item != NULL && i < li->startShowItem;
 	i++, item = item->next);
 
@@ -515,7 +518,7 @@ static void listboxDraw(newtComponent co)
 	    SLsmg_set_color(NEWT_COLORSET_SELLISTBOX);
 	else
 	    SLsmg_set_color(NEWT_COLORSET_LISTBOX);
-	    
+
 	SLsmg_write_nstring(item->text, li->curWidth);
 
     }
@@ -525,13 +528,15 @@ static void listboxDraw(newtComponent co)
 static struct eventResult listboxEvent(newtComponent co, struct event ev) {
     struct eventResult er;
     struct listbox * li = co->data;
-
+    struct items *item;
+    int i;
+    
     er.result = ER_IGNORED;
-	       
+
     if(ev.when == EV_EARLY || ev.when == EV_LATE) {
 	return er;
     }
-		       
+
     switch(ev.event) {
       case EV_KEYPRESS:
 	if (!li->isActive) break;
@@ -539,7 +544,8 @@ static struct eventResult listboxEvent(newtComponent co, struct event ev) {
 	switch(ev.u.key) {
 	  case ' ':
 	    if(!(li->flags & NEWT_FLAG_MULTIPLE)) break;
-	    newtListboxSelectItem(co, li->currItem, NEWT_FLAGS_TOGGLE);
+	    newtListboxSelectItem(co, li->boxItems[li->currItem].data,
+				  NEWT_FLAGS_TOGGLE);
 	    er.result = ER_SWALLOWED;
 	    /* We don't break here, because it is cool to be able to
 	       hold space to select a bunch of items in a list at once */
@@ -600,7 +606,7 @@ static struct eventResult listboxEvent(newtComponent co, struct event ev) {
 		li->startShowItem = li->numItems - li->curHeight;
 	    }
 	    li->currItem += li->curHeight;
-	    if(li->currItem > li->numItems) {
+	    if(li->currItem >= li->numItems) {
 		li->currItem = li->numItems - 1;
 	    }
 	    newtListboxRealSetCurrent(co);
@@ -623,10 +629,39 @@ static struct eventResult listboxEvent(newtComponent co, struct event ev) {
 	    er.result = ER_SWALLOWED;
 	    break;
 	  default:
-	    /* keeps gcc quiet */
+              if (ev.u.key < NEWT_KEY_EXTRA_BASE && isalpha(ev.u.key)) {
+		  for(i = 0, item = li->boxItems; item != NULL &&
+			  i < li->currItem; i++, item = item->next);
+
+		  if (item->text && (toupper(*item->text) == toupper(ev.u.key))) {
+		      item = item->next;
+		      i++;
+		  } else { 
+		      item = li->boxItems;
+		      i = 0;
+		  }
+		  while (item && item->text &&
+			 toupper(*item->text) != toupper(ev.u.key)) {
+		      item = item->next;
+		      i++;
+		  }
+		  if (item) {
+		      li->currItem = i;
+		      if(li->currItem < li->startShowItem ||
+			 li->currItem > li->startShowItem)
+			  li->startShowItem =
+			      li->currItem > li->numItems - li->curHeight ?
+			      li->startShowItem = li->numItems - li->curHeight :
+			      li->currItem;
+		      if(li->sb)
+			  newtScrollbarSet(li->sb, li->currItem + 1, li->numItems);
+		      newtListboxRealSetCurrent(co);
+		      er.result = ER_SWALLOWED;
+		  }
+	      }
 	}
 	break;
-	
+
       case EV_FOCUS:
 	li->isActive = 1;
 	listboxDraw(co);
@@ -638,6 +673,59 @@ static struct eventResult listboxEvent(newtComponent co, struct event ev) {
 	listboxDraw(co);
 	er.result = ER_SWALLOWED;
 	break;
+
+      case EV_MOUSE:
+	  /* if this mouse click was within the listbox, make the current
+	     item the item clicked on. */
+	/* Up scroll arrow */
+	if (li->sb &&
+	    ev.u.mouse.x == co->left + co->width - li->bdxAdjust - 1 &&
+	    ev.u.mouse.y == co->top + li->bdyAdjust) {
+	    if(li->numItems <= 0) break;
+	    if(li->currItem > 0) {
+		li->currItem--;
+		if(li->currItem < li->startShowItem)
+		    li->startShowItem = li->currItem;
+		if(li->sb)
+		    newtScrollbarSet(li->sb, li->currItem + 1, li->numItems);
+		listboxDraw(co);
+	    }
+	    if(co->callback) co->callback(co, co->callbackData);
+	    er.result = ER_SWALLOWED;
+	    break;
+	}
+	/* Down scroll arrow */
+	if (li->sb &&
+	    ev.u.mouse.x == co->left + co->width - li->bdxAdjust - 1 &&
+	    ev.u.mouse.y == co->top + co->height - li->bdyAdjust - 1) {
+	    if(li->numItems <= 0) break;
+	    if(li->currItem < li->numItems - 1) {
+		li->currItem++;
+		if(li->currItem > (li->startShowItem + li->curHeight - 1)) {
+		    li->startShowItem = li->currItem - li->curHeight + 1;
+		    if(li->startShowItem + li->curHeight > li->numItems)
+			li->startShowItem = li->numItems - li->curHeight;
+		}
+		if(li->sb)
+		    newtScrollbarSet(li->sb, li->currItem + 1, li->numItems);
+		listboxDraw(co);
+	    }
+	    if(co->callback) co->callback(co, co->callbackData);
+	    er.result = ER_SWALLOWED;
+	    break;
+	}
+	if ((ev.u.mouse.y >= co->top + li->bdyAdjust) &&
+	    (ev.u.mouse.y <= co->top + co->height - (li->bdyAdjust * 2)) &&
+	    (ev.u.mouse.x >= co->left + li->bdxAdjust) &&
+	    (ev.u.mouse.x <= co->left + co->width + (li->bdxAdjust * 2))) {
+	    li->currItem = li->startShowItem +
+		(ev.u.mouse.y - li->bdyAdjust - co->top);
+	    newtListboxRealSetCurrent(co);
+	    listboxDraw(co);
+	    if(co->callback) co->callback(co, co->callbackData);
+	    er.result = ER_SWALLOWED;
+	    break;
+	}
     }
 
     return er;
@@ -656,7 +744,8 @@ static void listboxDestroy(newtComponent co) {
 	item = nextitem;
     }
 
+    if (li->sb) li->sb->ops->destroy(li->sb);
+
     free(li);
     free(co);
 }
-
