@@ -32,12 +32,12 @@ static char ** currentHelpline = NULL;
 static int cursorRow, cursorCol;
 static int needResize;
 
-static const char * defaultHelpLine = 
+static const char * defaultHelpLine =
 "  <Tab>/<Alt-Tab> between elements   |  <Space> selects   |  <F12> next screen"
 ;
 
 const struct newtColors newtDefaultColorPalette = {
-	"white", "blue", 			/* root fg, bg */     
+	"white", "blue", 			/* root fg, bg */
 	"black", "lightgray",			/* border fg, bg */
 	"black", "lightgray",			/* window fg, bg */
 	"white", "black",			/* shadow fg, bg */
@@ -77,6 +77,7 @@ static const struct keymap keymap[] = {
 	{ "\033[4~",		NEWT_KEY_END, 		"kH" },
 
 	{ "\033[3~",		NEWT_KEY_DELETE,	"kl" },
+	{ "\033[2~", 		NEWT_KEY_INSERT,	NULL },
 
 	{ "\033\t",		NEWT_KEY_UNTAB,		NULL },
 
@@ -91,6 +92,11 @@ static const struct keymap keymap[] = {
 	{ "\033[[D",		NEWT_KEY_F4,		NULL },
 	{ "\033[[E",		NEWT_KEY_F5,		NULL },
 
+	{ "\033OP",		NEWT_KEY_F2,		NULL },
+	{ "\033OQ",		NEWT_KEY_F2,		NULL },
+	{ "\033OR",		NEWT_KEY_F2,		NULL },
+	{ "\033OS",		NEWT_KEY_F2,		NULL },
+
 	{ "\033[11~",		NEWT_KEY_F1,		NULL },
 	{ "\033[12~",		NEWT_KEY_F2,		NULL },
 	{ "\033[13~",		NEWT_KEY_F3,		NULL },
@@ -115,9 +121,11 @@ static const char * version = "Newt windowing library version " VERSION
 			"Written by Erik Troan\n";
 
 static newtSuspendCallback suspendCallback = NULL;
+static void * suspendCallbackData = NULL;
 
-void newtSetSuspendCallback(newtSuspendCallback cb) {
+void newtSetSuspendCallback(newtSuspendCallback cb, void * data) {
     suspendCallback = cb;
+    suspendCallbackData = data;
 }
 
 static void handleSigwinch(int signum) {
@@ -192,7 +200,7 @@ int newtInit(void) {
 
     SLsmg_init_smg();
     SLang_init_tty(0, 0, 0);
- 
+
     newtSetColors(newtDefaultColorPalette);
     /*initKeymap();*/
 
@@ -205,6 +213,8 @@ int newtInit(void) {
     SLsignal_intr(SIGWINCH, handleSigwinch);
     SLang_getkey_intr_hook = getkeyInterruptHook;
 
+
+
     return 0;
 }
 
@@ -224,25 +234,25 @@ void newtSetColors(struct newtColors colors) {
     SLtt_set_color(NEWT_COLORSET_SHADOW, "", colors.shadowFg, colors.shadowBg);
     SLtt_set_color(NEWT_COLORSET_TITLE, "", colors.titleFg, colors.titleBg);
     SLtt_set_color(NEWT_COLORSET_BUTTON, "", colors.buttonFg, colors.buttonBg);
-    SLtt_set_color(NEWT_COLORSET_ACTBUTTON, "", colors.actButtonFg, 
+    SLtt_set_color(NEWT_COLORSET_ACTBUTTON, "", colors.actButtonFg,
 			colors.actButtonBg);
-    SLtt_set_color(NEWT_COLORSET_CHECKBOX, "", colors.checkboxFg, 
+    SLtt_set_color(NEWT_COLORSET_CHECKBOX, "", colors.checkboxFg,
 			colors.checkboxBg);
-    SLtt_set_color(NEWT_COLORSET_ACTCHECKBOX, "", colors.actCheckboxFg, 
+    SLtt_set_color(NEWT_COLORSET_ACTCHECKBOX, "", colors.actCheckboxFg,
 			colors.actCheckboxBg);
     SLtt_set_color(NEWT_COLORSET_ENTRY, "", colors.entryFg, colors.entryBg);
     SLtt_set_color(NEWT_COLORSET_LABEL, "", colors.labelFg, colors.labelBg);
-    SLtt_set_color(NEWT_COLORSET_LISTBOX, "", colors.listboxFg, 
+    SLtt_set_color(NEWT_COLORSET_LISTBOX, "", colors.listboxFg,
 			colors.listboxBg);
-    SLtt_set_color(NEWT_COLORSET_ACTLISTBOX, "", colors.actListboxFg, 
+    SLtt_set_color(NEWT_COLORSET_ACTLISTBOX, "", colors.actListboxFg,
 			colors.actListboxBg);
-    SLtt_set_color(NEWT_COLORSET_TEXTBOX, "", colors.textboxFg, 
+    SLtt_set_color(NEWT_COLORSET_TEXTBOX, "", colors.textboxFg,
 			colors.textboxBg);
-    SLtt_set_color(NEWT_COLORSET_ACTTEXTBOX, "", colors.actTextboxFg, 
+    SLtt_set_color(NEWT_COLORSET_ACTTEXTBOX, "", colors.actTextboxFg,
 			colors.actTextboxBg);
-    SLtt_set_color(NEWT_COLORSET_HELPLINE, "", colors.helpLineFg, 
+    SLtt_set_color(NEWT_COLORSET_HELPLINE, "", colors.helpLineFg,
 			colors.helpLineBg);
-    SLtt_set_color(NEWT_COLORSET_ROOTTEXT, "", colors.rootTextFg, 
+    SLtt_set_color(NEWT_COLORSET_ROOTTEXT, "", colors.rootTextFg,
 			colors.rootTextBg);
 
     SLtt_set_color(NEWT_COLORSET_EMPTYSCALE, "", "black",
@@ -254,7 +264,7 @@ void newtSetColors(struct newtColors colors) {
 
     SLtt_set_color(NEWT_COLORSET_COMPACTBUTTON, "", colors.compactButtonFg,
 			colors.compactButtonBg);
-    
+
     SLtt_set_color(NEWT_COLORSET_ACTSELLISTBOX, "", colors.actSelListboxFg,
 		   colors.actSelListboxBg);
     SLtt_set_color(NEWT_COLORSET_SELLISTBOX, "", colors.selListboxFg,
@@ -277,7 +287,7 @@ int newtGetKey(void) {
 	}
 
 	if (key == NEWT_KEY_SUSPEND && suspendCallback)
-	    suspendCallback();
+	    suspendCallback(suspendCallbackData);
     } while (key == NEWT_KEY_SUSPEND);
 
     switch (key) {
@@ -336,7 +346,7 @@ int newtGetKey(void) {
        for later */
 
     chptr--;
-    while (chptr > buf) 
+    while (chptr > buf)
 	SLang_ungetkey(*chptr--);
 
     return *chptr;
@@ -355,7 +365,7 @@ void newtClearKeyBuffer(void) {
     }
 }
 
-int newtOpenWindow(int left, int top, int width, int height, 
+int newtOpenWindow(int left, int top, int width, int height,
 			  const char * title) {
     int j, row, col;
     int n;
@@ -427,7 +437,7 @@ int newtCenteredWindow(int width, int height, const char * title) {
 
     top = (SLtt_Screen_Rows - height) / 2;
 
-    /* I don't know why, but this seems to look better */ 
+    /* I don't know why, but this seems to look better */
     if ((SLtt_Screen_Rows % 2) && (top % 2)) top--;
 
     left = (SLtt_Screen_Cols - width) / 2;
@@ -439,7 +449,7 @@ int newtCenteredWindow(int width, int height, const char * title) {
 
 void newtPopWindow(void) {
     int j, row, col;
-    int n = 0;    
+    int n = 0;
 
     row = col = 0;
 
@@ -455,7 +465,7 @@ void newtPopWindow(void) {
     free(currentWindow->buffer);
     free(currentWindow->title);
 
-    if (currentWindow == windowStack) 
+    if (currentWindow == windowStack)
 	currentWindow = NULL;
     else
 	currentWindow--;
@@ -465,6 +475,14 @@ void newtPopWindow(void) {
     newtRefresh();
 }
 
+void newtGetWindowPos(int * x, int * y) {
+    if (currentWindow) {
+	*x = currentWindow->left;
+	*y = currentWindow->top;
+    } else
+	*x = *y = 0;
+}
+
 void newtGetrc(int * row, int * col) {
    *row = cursorRow;
    *col = cursorCol;
@@ -505,7 +523,7 @@ void newtClearBox(int left, int top, int width, int height) {
     SLsmg_fill_region(top, left, height, width, ' ');
 }
 
-#if 0    
+#if 0
 /* This doesn't seem to work quite right. I don't know why not, but when
    I rsh from an rxvt into a box and run this code, the machine returns
    console key's (\033[B) rather then xterm ones (\033OB). */
@@ -550,7 +568,7 @@ void newtRedrawHelpLine(void) {
     char * buf;
 
     SLsmg_set_color(NEWT_COLORSET_HELPLINE);
-   
+
     buf = alloca(SLtt_Screen_Cols + 1);
     memset(buf, ' ', SLtt_Screen_Cols);
     buf[SLtt_Screen_Cols] = '\0';
@@ -565,7 +583,7 @@ void newtRedrawHelpLine(void) {
 void newtPushHelpLine(const char * text) {
     if (!text)
 	text = defaultHelpLine;
-    
+
     if (currentHelpline)
 	(*(++currentHelpline)) = strdup(text);
     else {
@@ -588,7 +606,7 @@ void newtPopHelpLine(void) {
     newtRedrawHelpLine();
 }
 
-void newtDrawRootText(int row, int col, const char * text) {
+void newtDrawRootText(int col, int row, const char * text) {
     SLsmg_set_color(NEWT_COLORSET_ROOTTEXT);
 
     if (col < 0) {
@@ -596,9 +614,9 @@ void newtDrawRootText(int row, int col, const char * text) {
     }
 
     if (row < 0) {
-	col = SLtt_Screen_Cols + col;
+	row = SLtt_Screen_Rows + row;
     }
-   
+
     SLsmg_gotorc(row, col);
     SLsmg_write_string((char *)text);
 }
@@ -609,7 +627,7 @@ int newtSetFlags(int oldFlags, int newFlags, enum newtFlagsSense sense) {
 	return oldFlags | newFlags;
 
       case NEWT_FLAGS_RESET:
-	return oldFlags & (~newFlags); 
+	return oldFlags & (~newFlags);
 
       case NEWT_FLAGS_TOGGLE:
 	return oldFlags ^ newFlags;
