@@ -31,6 +31,9 @@ extern "C" {
 #define NEWT_COLORSET_ACTSELLISTBOX	23
 #define NEWT_COLORSET_SELLISTBOX	24
 
+#define NEWT_ARG_LAST			-100000
+#define NEWT_ARG_APPEND			-1
+
 struct newtColors {
     char * rootFg, * rootBg;
     char * borderFg, * borderBg;
@@ -62,12 +65,16 @@ enum newtFlagsSense { NEWT_FLAGS_SET, NEWT_FLAGS_RESET, NEWT_FLAGS_TOGGLE };
 #define NEWT_FLAG_HIDDEN 	(1 << 1)
 #define NEWT_FLAG_SCROLL 	(1 << 2)
 #define NEWT_FLAG_DISABLED 	(1 << 3)
-#define NEWT_FLAG_NOSCROLL 	(1 << 4)	/* for listboxes */
-#define NEWT_FLAG_DOBORDER	(1 << 5)
+/* OBSOLETE #define NEWT_FLAG_NOSCROLL 	(1 << 4)	for listboxes */
+#define NEWT_FLAG_BORDER	(1 << 5)
 #define NEWT_FLAG_WRAP		(1 << 6)
 #define NEWT_FLAG_NOF12		(1 << 7)
 #define NEWT_FLAG_MULTIPLE      (1 << 8)
 #define NEWT_FLAG_SELECTED	(1 << 9)
+#define NEWT_FLAG_CHECKBOX	(1 << 10)
+
+#define NEWT_FD_READ		(1 << 0)
+#define NEWT_FD_WRITE		(1 << 1)
 
 /* Backwards compatibility */
 #define NEWT_LISTBOX_RETURNEXIT NEWT_FLAG_RETURNEXIT
@@ -80,15 +87,15 @@ enum newtFlagsSense { NEWT_FLAGS_SET, NEWT_FLAGS_RESET, NEWT_FLAGS_TOGGLE };
 #define NEWT_TEXTBOX_SCROLL	NEWT_FLAG_SCROLL
 #define NEWT_FORM_NOF12		NEWT_FLAG_NOF12
 
-#define NEWT_FD_READ		(1 << 0)
-#define NEWT_FD_WRITE		(1 << 1)
+#define newtListboxAddEntry	newtListboxAppendEntry
+
 
 typedef struct newtComponent_struct * newtComponent;
 
 extern const struct newtColors newtDefaultColorPalette;
 
 typedef void (*newtCallback)(newtComponent, void *);
-typedef void (*newtSuspendCallback)(void);
+typedef void (*newtSuspendCallback)(void * data);
 
 int newtInit(void);
 int newtFinished(void);
@@ -105,12 +112,12 @@ void newtPopWindow(void);
 void newtSetColors(struct newtColors colors);
 void newtRefresh(void);
 void newtSuspend(void);
-void newtSetSuspendCallback(newtSuspendCallback cb);
+void newtSetSuspendCallback(newtSuspendCallback cb, void * data);
 void newtResume(void);
 void newtPushHelpLine(const char * text);
 void newtRedrawHelpLine(void);
 void newtPopHelpLine(void);
-void newtDrawRootText(int row, int col, const char * text);
+void newtDrawRootText(int col, int row, const char * text);
 void newtBell(void);
 
 /* Components */
@@ -120,6 +127,10 @@ newtComponent newtButton(int left, int top, const char * text);
 newtComponent newtCheckbox(int left, int top, const char * text, char defValue,
 			   const char * seq, char * result);
 char newtCheckboxGetValue(newtComponent co);
+void newtCheckboxSetValue(newtComponent co, char value);
+void newtCheckboxSetFlags(newtComponent co, int flags, enum newtFlagsSense sense);
+
+    
 newtComponent newtRadiobutton(int left, int top, const char * text, int isDefault,
 			      newtComponent prevButton);
 newtComponent newtRadioGetCurrent(newtComponent setMember);
@@ -139,12 +150,11 @@ newtComponent newtListbox(int left, int top, int height, int flags);
 void * newtListboxGetCurrent(newtComponent co);
 void newtListboxSetCurrent(newtComponent co, int num);
 void newtListboxSetCurrentByKey(newtComponent co, void * key);
-void newtListboxSetText(newtComponent co, int num, const char * text);
 void newtListboxSetEntry(newtComponent co, int num, const char * text);
 void newtListboxSetWidth(newtComponent co, int width);
-/* return the data passed to AddEntry */
 void newtListboxSetData(newtComponent co, int num, void * data);
-int newtListboxAddEntry(newtComponent co, const char * text, const void * data);
+int newtListboxAppendEntry(newtComponent co, const char * text, 
+			   const void * data);
 /* Send the key to insert after, or NULL to insert at the top */
 int newtListboxInsertEntry(newtComponent co, const char * text, const void * data, void * key);
 int newtListboxDeleteEntry(newtComponent co, void * data);
@@ -153,13 +163,25 @@ void newtListboxGetEntry(newtComponent co, int num, char **text, void **data);
 /* Returns an array of data pointers from items, last element is NULL */
 void **newtListboxGetSelection(newtComponent co, int *numitems);
 void newtListboxClearSelection(newtComponent co);
-void newtListboxSelectItem(newtComponent co, int item,
+void newtListboxSelectItem(newtComponent co, const void * key,
 	enum newtFlagsSense sense);
 
+newtComponent newtCheckboxTree(int left, int top, int height, int flags);
+newtComponent newtCheckboxTreeMulti(int left, int top, int height, char *seq, int flags);
+void ** newtCheckboxTreeGetSelection(newtComponent co, int *numitems);
+void ** newtCheckboxTreeGetMultiSelection(newtComponent co, int *numitems, char seqnum);
+/* last item is NEWT_ARG_LAST for all of these */
+int newtCheckboxTreeAddItem(newtComponent co, 
+			    const char * text, const void * data,
+			    int flags, int index, ...);
+int newtCheckboxTreeAddArray(newtComponent co, 
+			    const char * text, const void * data,
+			    int flags, int * indexes);
+int * newtCheckboxTreeFindItem(newtComponent co, void * data);
     
 newtComponent newtTextboxReflowed(int left, int top, char * text, int width,
 				  int flexDown, int flexUp, int flags);
-newtComponent newtTextbox(int left, int top, int with, int height, int flags);
+newtComponent newtTextbox(int left, int top, int width, int height, int flags);
 void newtTextboxSetText(newtComponent co, const char * text);
 void newtTextboxSetHeight(newtComponent co, int height);
 int newtTextboxGetNumLines(newtComponent co);
@@ -196,6 +218,7 @@ newtComponent newtEntry(int left, int top, const char * initialValue, int width,
 void newtEntrySet(newtComponent co, const char * value, int cursorAtEnd);
 void newtEntrySetFilter(newtComponent co, newtEntryFilter filter, void * data);
 char * newtEntryGetValue(newtComponent co);
+void newtEntrySetFlags(newtComponent co, int flags, enum newtFlagsSense sense);
 
 newtComponent newtScale(int left, int top, int width, long long fullValue);
 void newtScaleSet(newtComponent co, unsigned long long amount);
@@ -226,6 +249,7 @@ void newtFormDestroy(newtComponent form);
 #define NEWT_KEY_UNTAB			NEWT_KEY_EXTRA_BASE + 10
 #define NEWT_KEY_PGUP			NEWT_KEY_EXTRA_BASE + 11
 #define NEWT_KEY_PGDN			NEWT_KEY_EXTRA_BASE + 12
+#define NEWT_KEY_INSERT			NEWT_KEY_EXTRA_BASE + 13
 
 #define NEWT_KEY_F1			NEWT_KEY_EXTRA_BASE + 101
 #define NEWT_KEY_F2			NEWT_KEY_EXTRA_BASE + 102
@@ -270,6 +294,7 @@ void newtGridSetField(newtGrid grid, int col, int row,
 		      int padTop, int padRight, int padBottom, int anchor,
 		      int flags);
 void newtGridPlace(newtGrid grid, int left, int top);
+#define newtGridDestroy newtGridFree
 void newtGridFree(newtGrid grid, int recurse);
 void newtGridGetSize(newtGrid grid, int * width, int * height);
 void newtGridWrappedWindow(newtGrid grid, char * title);
