@@ -72,7 +72,7 @@ newtComponent newtTextboxReflowed(int left, int top, char * text, int width,
     reflowedText = newtReflowText(text, width, flexDown, flexUp,
 				  &actWidth, &actHeight);
     
-    co = newtTextbox(-1, -1, actWidth, actHeight, NEWT_TEXTBOX_WRAP);
+    co = newtTextbox(left, top, actWidth, actHeight, NEWT_FLAG_WRAP);
     newtTextboxSetText(co, reflowedText);
     free(reflowedText);
 
@@ -118,6 +118,7 @@ static char * expandTabs(const char * text) {
     char * buf, * dest;
     const char * src;
     int bufUsed = 0;
+    int linePos = 0;
     int i;
 
     buf = malloc(bufAlloced + 1);
@@ -128,10 +129,15 @@ static char * expandTabs(const char * text) {
 	    dest = buf + bufUsed;
 	}
 	if (*src == '\t') {
-	    i = 8 - (bufUsed & 8);
+	    i = 8 - (linePos & 8);
 	    memset(dest, ' ', i);
-	    dest += i, bufUsed += i;
+	    dest += i, bufUsed += i, linePos += i;
 	} else {
+	    if (*src == '\n')
+		linePos = 0;
+	    else
+		linePos++;
+
 	    *dest++ = *src;
 	    bufUsed++;
 	}
@@ -174,8 +180,11 @@ static void doReflow(const char * text, char ** resultPtr, int width,
 	    } else {
 		chptr = text + width - 1;
 		while (chptr > text && !isspace(*chptr)) chptr--;
-		while (isspace(*chptr)) chptr--;
+		while (chptr > text && isspace(*chptr)) chptr--;
 		chptr++;
+		
+		if (chptr-text == 1 && !isspace(*chptr))
+		  chptr = text + width - 1;
 
 		if (chptr > text)
 		    howbad += width - (chptr - text) + 1;
@@ -185,7 +194,10 @@ static void doReflow(const char * text, char ** resultPtr, int width,
 		    height++;
 		}
 
-		text = chptr + 1;
+		if (isspace(*chptr))
+		    text = chptr + 1;
+		else
+		  text = chptr;
 		while (isspace(*text)) text++;
 	    }
 	}
@@ -323,8 +335,10 @@ static struct eventResult textboxEvent(newtComponent co,
 
 	  case NEWT_KEY_PGDN:
 	    tb->topLine += co->height;
-	    if (tb->topLine > (tb->numLines - co->height))
+	    if (tb->topLine > (tb->numLines - co->height)) {
 		tb->topLine = tb->numLines - co->height;
+		if (tb->topLine < 0) tb->topLine = 0;
+	    }
 	    textboxDraw(co);
 	    er.result = ER_SWALLOWED;
 	    break;
@@ -337,7 +351,23 @@ static struct eventResult textboxEvent(newtComponent co,
 	    break;
 	}
     }
-
+    if (ev.when == EV_EARLY && ev.event == EV_MOUSE && tb->sb) {
+	/* Top scroll arrow */
+	if (ev.u.mouse.x == co->width && ev.u.mouse.y == co->top) {
+	    if (tb->topLine) tb->topLine--;
+	    textboxDraw(co);
+	    
+	    er.result = ER_SWALLOWED;
+	}
+	/* Bottom scroll arrow */
+	if (ev.u.mouse.x == co->width &&
+	    ev.u.mouse.y == co->top + co->height - 1) {
+	    if (tb->topLine < (tb->numLines - co->height)) tb->topLine++;
+	    textboxDraw(co);
+	    
+	    er.result = ER_SWALLOWED;
+	}
+    }
     return er;
 }
 
