@@ -23,7 +23,7 @@ static void * newtvwindow(char * title, char * button1, char * button2,
 	if (buf) free(buf);
 	buf = malloc(size);
 	i = vsnprintf(buf, size, message, args);
-    } while (i == size);
+    } while (i >= size || i == -1);
 
     flowedText = newtReflowText(buf, 35, 5, 5, &width, &height);
     if (height > 6) {
@@ -152,7 +152,7 @@ int newtWinMenu(char * title, char * text, int suggestedWidth, int flexDown,
     needScroll = i > maxListHeight;
 
     listbox = newtListbox(-1, -1, maxListHeight, 
-		  (needScroll ? 0 : NEWT_FLAG_NOSCROLL) | NEWT_FLAG_RETURNEXIT);
+		  (needScroll ? NEWT_FLAG_SCROLL : 0) | NEWT_FLAG_RETURNEXIT);
     for (i = 0; items[i]; i++) {
 	newtListboxAddEntry(listbox, items[i], (void *) i);
     }
